@@ -480,7 +480,7 @@ class IsolatedAptCache(object):
         logger.debug("Writing apt configs")
         self.tempdir = tempfile.mkdtemp(prefix="hwpack-apt-cache-")
         dirs = ["var/lib/dpkg",
-                "etc/apt/",
+                "etc/apt/sources.list.d",
                 "var/cache/apt/archives/partial",
                 "var/lib/apt/lists/partial",
                ]
@@ -724,4 +724,5 @@ class PackageFetcher(object):
                     "The item %r could not be fetched: %s" %
                     (acqfile.destfile, acqfile.error_text))
             result_package.content = open(destfile)
+            result_package._file_path = destfile
         return fetched.values()
