@@ -58,7 +58,7 @@ def get_args_parser():
         '--dev', required=True, dest='board', choices=KNOWN_BOARDS,
         help='Generate an SD card or image for the given board.')
     parser.add_argument(
-        '--rootfs', default='ext3', choices=['ext2', 'ext3', 'ext4', 'btrfs'],
+        '--rootfs', default='ext4', choices=['ext2', 'ext3', 'ext4', 'btrfs'],
         help='Type of filesystem to use for the rootfs')
     parser.add_argument(
         '--rfs_label', default='rootfs',
@@ -96,7 +96,7 @@ def get_args_parser():
         '--hwpack-force-yes', action='store_true',
         help='Pass --force-yes to linaro-hwpack-install')
     parser.add_argument(
-        '--image_size', default='2G',
+        '--image_size', default='3G',
         help=('The image size, specified in mega/giga bytes (e.g. 3000M or '
               '3G); use with --image_file only'))
     parser.add_argument(
@@ -119,13 +119,26 @@ def get_args_parser():
         '--align-boot-part', dest='should_align_boot_part',
         action='store_true',
         help='Align boot partition too (might break older x-loaders).')
+    parser.add_argument(
+        '--nocheck-mmc', dest='nocheck_mmc',
+        action='store_true',
+        help='Assume yes to the question "Are you 100%% sure, on selecting [mmc]"')
+    
     return parser
 
 def get_android_args_parser():
     """Get the ArgumentParser for the arguments given on the command line."""
     parser = argparse.ArgumentParser()
+    group = parser.add_mutually_exclusive_group(required=True)
+    group.add_argument(
+        '--mmc', dest='device', help='The storage device to use.')
+    group.add_argument(
+        '--image_file', dest='device',
+        help='File where we should write the image file.')
     parser.add_argument(
-        '--mmc', required=True, dest='device', help='The storage device to use.')
+        '--image_size', default='2G',
+        help=('The image size, specified in mega/giga bytes (e.g. 3000M or '
+              '3G); use with --image_file only'))
     parser.add_argument(
         '--dev', required=True, dest='board', choices=ANDROID_KNOWN_BOARDS,
         help='Generate an SD card or image for the given board.')
